import json

none = "d3043820717d74d9a17694c176d39733"

# region EMR
class DeploymentAction:

	def __init__(
		self,
		action_type=none,
		should_handle_all_batches=none,
		draining_timeout=none,
		should_decrement_target_capacity=none):
		"""

		:type action_type: str
		:type should_handle_all_batches: bool
		:type draining_timeout: int
		:type should_decrement_target_capacity: bool
		"""
		self.action_type = action_type
		self.should_handle_all_batches = should_handle_all_batches
		self.draining_timeout = draining_timeout
		self.should_decrement_target_capacity = should_decrement_target_capacity

# endregion


class DeploymentActionRequest:
    def __init__(self, roll):
        self.roll = roll

    def toJSON(self):
        return json.dumps(self.roll, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)










import json
import os
import re

import requests
import yaml

from spotinst_sdk import aws_elastigroup
from spotinst_sdk import spotinst_functions
from spotinst_sdk import spotinst_emr
from spotinst_sdk import spotinst_stateful
from spotinst_sdk import spotinst_blue_green_deployment
from spotinst_sdk import spotinst_deployment_action
from spotinst_sdk import spotinst_asg

VAR_SPOTINST_SHARED_CREDENTIALS_FILE = 'SPOTINST_SHARED_CREDENTIALS_FILE'
VAR_SPOTINST_PROFILE = 'SPOTINST_PROFILE'
VAR_SPOTINST_TOKEN = 'SPOTINST_TOKEN'
VAR_SPOTINST_ACCOUNT = 'SPOTINST_ACCOUNT'

DEFAULT_PROFILE = 'default'
DEFAULT_CREDENTIALS_FILE = os.path.join(
    os.path.expanduser("~"), '.spotinst', 'credentials')

version = {}
with open(os.path.join(os.path.dirname(__file__), "./version.py")) as fp:
    exec(fp.read(), version)

_SpotinstClient__spotinst_sdk_python_agent_name = 'spotinst-sdk-python'
_SpotinstClient__spotinst_sdk_user_agent = '{}/{}'.format(
    _SpotinstClient__spotinst_sdk_python_agent_name, version['__version__'])


class SpotinstClient:
    __account_id_key = "accountId"
    __base_url = "https://api.spotinst.io/aws/ec2"
    __base_elastigroup_url = "https://api.spotinst.io/aws/ec2/group"
    __base_emr_url = "https://api.spotinst.io/aws/emr/mrScaler"
    __base_functions_url = "https://api.spotinst.io/functions"
    __base_stateful_url = "https://api.spotinst.io/aws/ec2/statefulMigrationGroup"
    __base_kube_url = "https://api.spotinst.io/mcs/kubernetes/cluster"
    __base_saving_url = "https://api.spotinst.io/aws/potentialSavings"

    camel_pat = re.compile(r'([A-Z])')
    under_pat = re.compile(r'_([a-z])')

    # region Constructor
    def __init__(self, auth_token=None,
                 account_id=None,
                 profile=None,
                 credentials_file=None,
                 print_output=True,
                 user_agent=None):
        """

        :type auth_token: str
        :type account_id: str
        :type profile: str
        :type credentials_file: str
        :type print_output: bool
        :type user_agent: str
        """

        if not auth_token:
            self.load_credentials(profile, credentials_file)
        else:
            self.auth_token = auth_token
            self.account_id = account_id

        self.should_print_output = print_output
        self.user_agent = user_agent


    # region EMR
    def create_emr(self, emr):
        emr = spotinst_emr.EMRCreationRequest(emr)

        excluded_group_dict = self.exclude_missing(json.loads(emr.toJSON()))
        
        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)
        
        group_response = self.send_post(
            body=body_json,
            url=self.__base_emr_url,
            entity_name='emr')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    # endregion

    # region Kubernetes

    def get_kubernetes_cluster_cost(self, custer_id, from_date, to_date):        
        geturl = self.__base_kube_url + "/" + custer_id + "/costs"
        query_params = self.build_query_params_with_input({"toDate":to_date, "fromDate":from_date})

        result = self.send_get(url=geturl, query_params=query_params, entity_name='kubernetes')

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    # endregion



    # region Elastigroup
    def create_elastigroup(self, group):        
        group = aws_elastigroup.ElastigroupCreationRequest(group)

        excluded_group_dict = self.exclude_missing(json.loads(group.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        self.print_output(body_json)

        group_response = self.send_post(
            body=body_json,
            url=self.__base_elastigroup_url,
            entity_name='elastigroup')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def scale_elastigroup_up(self, group_id, adjustment):
        query_params = dict({"adjustment": adjustment})
        content = self.send_put_with_params(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/scale/up",
            entity_name='elastigroup (scale up)',
            body=None,
            user_query_params=query_params)

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def scale_elastigroup_down(self, group_id, adjustment):
        query_params = dict({"adjustment": adjustment})
        content = self.send_put_with_params(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/scale/down",
            entity_name='elastigroup (scale down)',
            body=None,
            user_query_params=query_params)

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def update_elastigroup(self, group_update, group_id):

        group = aws_elastigroup.ElastigroupUpdateRequest(group_update)

        excluded_group_update_dict = self.exclude_missing(
            json.loads(group.toJSON()))

        formatted_group_update_dict = self.convert_json(
            excluded_group_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_update_dict)

        self.print_output(body_json)

        group_response = self.send_put(
            self.__base_elastigroup_url +
            "/" +
            group_id,
            entity_name='elastigroup',
            body=body_json)

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def delete_elastigroup(self, group_id):
        delurl = self.__base_elastigroup_url + "/" + group_id
        response = self.send_delete(url=delurl, entity_name='elastigroup')
        return response

    def delete_elastigroup_with_deallocation(
            self, group_id, stateful_deallocation):
        delurl = self.__base_elastigroup_url + "/" + group_id

        deletion_request = aws_elastigroup.ElastigroupDeletionRequest(
            stateful_deallocation)

        excluded_deletion_dict = self.exclude_missing(
            json.loads(deletion_request.toJSON()))
        formatted_deletion_dict = self.convert_json(
            excluded_deletion_dict, self.underscore_to_camel)
        body_json = json.dumps(formatted_deletion_dict)

        response = self.send_delete_with_body(
            body=body_json, url=delurl, entity_name='elastigroup')

        return response

    def get_elastigroup(self, group_id):
        geturl = self.__base_elastigroup_url + "/" + group_id
        result = self.send_get(url=geturl, entity_name='elastigroup')

        formatted_response = self.convert_json(
            result, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_elastigroups(self):
        content = self.send_get(
            url=self.__base_elastigroup_url,
            entity_name='elastigroup')
        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def get_elastigroup_active_instances(self, group_id):
        content = self.send_get(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/status",
            entity_name='active instances')
        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def get_elastigroup_activity(self, group_id, start_date):
        query_params = self.build_query_params_with_input({"fromDate":start_date})

        content = self.send_get(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/events",
            query_params=query_params,
            entity_name='active events')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]

    def roll_group(self, group_id, group_roll):
        group_roll_request = aws_elastigroup.ElastigroupRollRequest(
            group_roll=group_roll)


        excluded_group_roll_dict = self.exclude_missing(
            json.loads(group_roll_request.toJSON()))

        formatted_group_roll_dict = self.convert_json(
            excluded_group_roll_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_roll_dict)

        roll_response = self.send_put(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/roll",
            body=body_json,
            entity_name='roll')

        formatted_response = self.convert_json(
            roll_response, self.camel_to_underscore)

        retVal = formatted_response["response"]

        return retVal

    def get_all_group_deployment(self, group_id):
        content = self.send_get(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/roll",
            entity_name='roll')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]


    def get_deployment_status(self, group_id, roll_id):
        content = self.send_get(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/roll/"+
                str(roll_id),
            entity_name='roll')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def stop_deployment(self, group_id, roll_id):
        content = self.send_put(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/roll/"+
                str(roll_id),
            body=json.dumps(dict(roll=dict(status="STOPPED"))),
            entity_name='roll')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)

        return formatted_response["response"]

    def create_deployment_action(self, group_id, roll_id, deployment_action):
        deployment_action_request = spotinst_deployment_action.DeploymentActionRequest(deployment_action)

        deployment_action_dict = self.exclude_missing(
            json.loads(deployment_action_request.toJSON()))

        formatted_group = self.convert_json(
            deployment_action_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group)

        detach_response = self.send_post(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/roll/"+
                str(roll_id),
            body=body_json,
            entity_name='roll')

        formatted_response = self.convert_json(
            detach_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal

    def get_instance_type_by_region(self, region):
        query_params = dict(region=region)
        response = self.send_get(
            url=self.__base_url+
            "/spotType",
            query_params=query_params,
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"]

    def lock_instance(self, instance_id, lock_time=None):
        query_params= dict(ttlInMinutes=lock_time)

        response = self.send_post(
            url=self.__base_url +
            "/instance/" +
            instance_id +
            "/lock",
            query_params=query_params,
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["status"]

    def unlock_instance(self, instance_id):
        response = self.send_post(
            url=self.__base_url +
            "/instance/" +
            instance_id +
            "/unlock",
            entity_name="instance"
        )
        
        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["status"]

    def enter_instance_standby(self, instance_id):
        response = self.send_post(
            url=self.__base_url + 
            "/instance/" + 
            instance_id +
            "/standby/enter",
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["status"]

    def exit_instance_standby(self, instance_id):
        response = self.send_post(
            url=self.__base_url + 
            "/instance/" + 
            instance_id +
            "/standby/exit",
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["status"]

    def get_instance_status(self, instance_id):
        response = self.send_get(
            url=self.__base_url + 
            "/instance/" +
            instance_id,
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_instance_healthiness(self, group_id):
        response = self.send_get(
            url=self.__base_elastigroup_url +
             "/" + group_id + 
            "/instanceHealthiness",
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"][0]  


    def create_instance_signal(self, instance_id, signal):
        body = dict(instanceId=instance_id, signal=signal)

        response = self.send_post(
            url= self.__base_url + 
            "/instance/signal",
            body=body,
            entity_name="instance"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["status"]  

    def get_cost_per_account(self, to_date=None, from_date=None):
        query_params=dict(toDate=to_date, fromDate=from_date)

        response = self.send_get(
            url="https://api.spotinst.io/aws/costs",
            query_params=query_params,
            entity_name="cost"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"]  

    def get_cost_per_elastigroup(self, group_id, to_date=None, from_date=None):
        query_params=dict(toDate=to_date, fromDate=from_date)

        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/costs",
            query_params=query_params,
            entity_name="cost"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"]   

    def get_group_detailed_cost(self, group_id, to_date=None, from_date=None):
        query_params=dict(toDate=to_date, fromDate=from_date)

        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id +
            "/costs/detailed",
            query_params=query_params,
            entity_name="cost"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"] 

    def get_potential_savings(self):
        response = self.send_get(
            url="https://api.spotinst.io/aws/potentialSavings",
            entity_name="saving"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"] 

    def get_instance_potential_savings(self, instance_ids, region):
        instance_str = ""

        for instance in instance_ids:
            instance_str += instance + "," 
        
        query_params = dict(region=region, instanceIds=instance_str)

        response = self.send_get(
            url="https://api.spotinst.io/aws/instancePotentialSavings",
            query_params=query_params,
            entity_name="saving"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"] 

    def list_suspended_scaling_policies(self, group_id):
        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id + 
            "/scale/suspensions",
            entity_name="scaling policies"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"]   

    def suspend_scaling_policies(self, group_id, policy_name):
        query_params = dict(policyName=policy_name)

        response = self.send_post(
            url=self.__base_elastigroup_url +
            "/" + group_id + 
            "/scale/suspendPolicy",
            query_params=query_params,
            entity_name="scaling policies"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def resume_suspended_scaling_policies(self, group_id, policy_name):
        query_params = dict(policyName=policy_name)

        response = self.send_post(
            url=self.__base_elastigroup_url +
            "/" + group_id + 
            "/scale/resumePolicy",
            query_params=query_params,
            entity_name="scaling policies"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["status"]

    def list_suspended_process(self, group_id):
        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id + 
            "/suspension",
            entity_name="suspend process"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"]

    def suspend_process(self, group_id, processes, suspensions):
        body = dict(suspensions=suspensions, processes=processes)
        
        response = self.send_post(
            url=self.__base_elastigroup_url +
            "/" + group_id + 
            "/suspension",
            body=body,
            entity_name="suspend process"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response["response"]["items"]

    def remove_suspended_process(self, group_id, processes):
        body = dict(processes=processes)

        response = self.send_delete(
            url=self.__base_elastigroup_url +
            "/" + group_id + 
            "/suspension",
            body=body,
            entity_name="suspend process"
        )

        formatted_response = self.convert_json(
            response, self. camel_to_underscore)

        return formatted_response

    def detach_elastigroup_instances(self, group_id, detach_configuration):

        group_detach_request = aws_elastigroup.ElastigroupDetachInstancesRequest(
            detach_configuration=detach_configuration)

        excluded_group_detach_dict = self.exclude_missing(
            json.loads(group_detach_request.toJSON()))

        formatted_group_detach_dict = self.convert_json(
            excluded_group_detach_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_detach_dict)

        detach_response = self.send_put(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/detachInstances",
            body=body_json,
            entity_name='detach')

        formatted_response = self.convert_json(
            detach_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal

    def import_stateful_instance(self, stateful_instance):
        stateful_instance = spotinst_stateful.StatefulImportRequest(stateful_instance)

        excluded_group_dict = self.exclude_missing(json.loads(stateful_instance.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)
        
        group_response = self.send_post(
            body=body_json,
            url=self.__base_stateful_url,
            entity_name='import stateful instance')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def get_stateful_import_status(self, stateful_migration_id):
        content = self.send_get(
            url=self.__base_stateful_url +
                "/" +
                str(stateful_migration_id),
            entity_name='get stateful import status')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)

        return formatted_response["response"]["items"] 

    def delete_stateful_import(self, stateful_migration_id):
        content = self.send_delete(
            url=self.__base_stateful_url +
                "/" +
                str(stateful_migration_id),
            entity_name='delete stateful import')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response

    def deallocate_stateful_instance(self, group_id, stateful_instance_id):
        content = self.send_put(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/statefulInstance/" +
                str(stateful_instance_id +
                "/deallocate"),
            entity_name='deallocate stateful instance')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]

    def recycle_stateful_instance(self, group_id, stateful_instance_id):
        content = self.send_put(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/statefulInstance/" +
                str(stateful_instance_id +
                "/recycle"),
            entity_name='recycle stateful instance')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]


    def get_stateful_instances(self, group_id):
        content = self.send_get(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/statefulInstance",
            entity_name='get stateful instance')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]["items"]           

    def resume_stateful_instance(self, group_id, stateful_instance_id):
        content = self.send_put(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/statefulInstance/" +
                str(stateful_instance_id +
                "/resume"),
            entity_name='resume stateful instance')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)
        return formatted_response["response"]

    def pause_stateful_instance(self, group_id, stateful_instance_id):
        content = self.send_put(
            url=self.__base_elastigroup_url +
                "/" +
                str(group_id) +
                "/statefulInstance/" +
                str(stateful_instance_id) +
                "/pause",
            entity_name='pause stateful instance')

        formatted_response = self.convert_json(
            content, self.camel_to_underscore)

        return formatted_response["response"]



    def beanstalk_maintenance_status(self, group_id):
        
        status_response = self.send_get(
            url=self.__base_elastigroup_url+
            "/" +
            str(group_id) + 
            "/beanstalk/maintenance/status",
            entity_name="beanstalk maintenance start")

        formatted_response = self.convert_json(
            status_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal


    def beanstalk_maintenance_start(self, group_id):

        start_response = self.send_put(
            url=self.__base_elastigroup_url+
            "/" +
            str(group_id) + 
            "/beanstalk/maintenance/start",
            body={},
            entity_name="beanstalk maintenance start")

        formatted_response = self.convert_json(
            start_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal

    def beanstalk_maintenance_finish(self, group_id):   

        finish_response = self.send_put(
            url=self.__base_elastigroup_url+
            "/" +
            str(group_id) + 
            "/beanstalk/maintenance/finish",
            body={},
            entity_name="beanstalk maintenance start")

        formatted_response = self.convert_json(
            finish_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["status"]

        return retVal

    def beanstalk_import(self, region, env_id=None, env_name=None):
        query_params = dict(region=region, environmentId=env_id, environmentName=env_name)

        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/beanstalk/import",
            query_params=query_params,
            entity_name="beanstalk import"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def beanstalk_reimport(self, group_id):
        response = self.send_put(
            url=self.__base_elastigroup_url +
            "/" + str(group_id) + 
            "/beanstalk/reimport",
            entity_name="beanstalk reimport"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def import_asg(self, region, asg_name, asg, dry_run=None):
        query_params = dict(region=region, autoScalingGroupName=asg_name, dryRun=dry_run)

        asg = spotinst_asg.ImportASGRequest(asg)

        excluded_group_dict = self.exclude_missing(json.loads(asg.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)
        
        response = self.send_post(
            body=body_json,
            url=self.__base_elastigroup_url+
            "/autoScalingGroup/import",
            query_params=query_params,
            entity_name='import asg')

        print(json.dumps(response))

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def get_activity_events(self, group_id, from_date):
        query_params = dict(fromDate=from_date)

        response = self.send_get(
            url=self.__base_elastigroup_url +
            "/" + group_id + "/events",
            query_params=query_params,
            entity_name="activity groups"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"]

        return retVal

    def ami_backup(self, group_id):
        response = self.send_post(
            url=self.__base_elastigroup_url +
            "/" + group_id + "/amiBackup",
            entity_name="ami backup"
        )

        print(json.dumps(response))

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response

        return retVal["response"]["status"]

    def create_blue_green_deployment(self, group_id, blue_green_deployment):
        blue_green_deployment = spotinst_blue_green_deployment.BlueGreenDeploymentRequest(blue_green_deployment)

        excluded_group_dict = self.exclude_missing(json.loads(blue_green_deployment.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        group_response = self.send_post(
            body=body_json,
            url=self.__base_elastigroup_url + "/" + group_id + "/codeDeploy/blueGreenDeployment",
            entity_name='create b/g deployment')

        print(group_response)

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def get_blue_green_deployment(self, group_id):
        response = self.send_get(
            url= self.__base_elastigroup_url + "/" + group_id + "/codeDeploy/blueGreenDeployment",
            entity_name="get b/g deployment")

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def stop_blue_green_deployment(self, group_id, deployment_id):
        response = self.send_put(
            url=self.__base_elastigroup_url + "/" + group_id + "/codeDeploy/blueGreenDeployment/" + deployment_id + "/stop",
            entity_name="stop b/g deployment")

        print(response)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        print(formatted_response)

        retVal = formatted_response["response"]

        return retVal

    # endregion

    # region Functions
    def create_application(self, app):

        app = spotinst_functions.ApplicationCreationRequest(app)

        excluded_group_dict = self.exclude_missing(json.loads(app.toJSON()))

        formatted_app_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_app_dict)

        self.print_output(body_json)

        app_response = self.send_post(
            body=body_json,
            url=self.__base_functions_url +
            '/application',
            entity_name='application')

        formatted_response = self.convert_json(
            app_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def create_environment(self, env):

        env = spotinst_functions.EnvironmentCreationRequest(env)

        excluded_env_dict = self.exclude_missing(json.loads(env.toJSON()))

        formatted_env_dict = self.convert_json(
            excluded_env_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_env_dict)

        self.print_output(body_json)

        env_response = self.send_post(
            body=body_json,
            url=self.__base_functions_url +
            '/environment',
            entity_name='environment')

        formatted_response = self.convert_json(
            env_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    def create_function(self, fx):

        fx = spotinst_functions.FunctionCreationRequest(
            fx, self.should_print_output)

        excluded_fx_dict = self.exclude_missing(json.loads(fx.toJSON()))

        formatted_fx_dict = self.convert_json(
            excluded_fx_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_fx_dict)

        formatted_fx_dict['function']['code']['source'] = 'INLINE_BASE64_SOURCE_CODE'
        self.print_output(json.dumps(formatted_fx_dict))

        fx_response = self.send_post(
            body=body_json,
            url=self.__base_functions_url +
            '/function',
            entity_name='function')

        formatted_response = self.convert_json(
            fx_response, self.camel_to_underscore)

        retVal = formatted_response["response"]["items"][0]

        return retVal

    # endregion

    # region Utils
    def print_output(self, output):
        if self.should_print_output is True:
            print(output)

    def send_get(self, url,entity_name,query_params=None):
        agent = self.resolve_user_agent()
        query_params = query_params or self.build_query_params()
        headers = dict(
            {
                'User-Agent': agent,
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + self.auth_token
            }
        )

        self.print_output("Sending get request to spotinst API.")
        result = requests.get(url, params=query_params, headers=headers)

        if result.status_code == requests.codes.ok:
            self.print_output("Success")
            data = json.loads(result.content.decode('utf-8'))
            return data
        else:
            self.handle_exception("getting {}".format(entity_name), result)

    def send_delete(self, url, entity_name, body=None):
        agent = self.resolve_user_agent()
        query_params = self.build_query_params()
        headers = dict(
            {
                'User-Agent': agent,
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + self.auth_token
            }
        )

        self.print_output("Sending deletion request to spotinst API.")
        result = requests.delete(url, params=query_params, body=body, headers=headers)

        if result.status_code == requests.codes.ok:
            self.print_output("Success")
            return True
        else:
            self.handle_exception("deleting {}".format(entity_name), result)

    def send_delete_with_body(self, body, url, entity_name):
        agent = self.resolve_user_agent()
        query_params = self.build_query_params()
        headers = dict(
            {
                'User-Agent': agent,
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + self.auth_token
            }
        )

        self.print_output("Sending deletion request to spotinst API.")
        result = requests.delete(
            url,
            params=query_params,
            headers=headers,
            data=body)

        if result.status_code == requests.codes.ok:
            self.print_output("Success")
            return True
        else:
            self.handle_exception("deleting {}".format(entity_name), result)

    def send_post(self, url, entity_name, body=None, query_params=None):
        agent = self.resolve_user_agent()
        query_params = query_params or self.build_query_params()
        headers = dict(
            {
                'User-Agent': agent,
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + self.auth_token
            }
        )

        self.print_output("Sending post request to spotinst API.")
        result = requests.post(
            url,
            params=query_params,
            data=body,
            headers=headers)

        if result.status_code == requests.codes.ok:
            self.print_output("Success")
            data = json.loads(result.content.decode('utf-8'))
            return data
        else:
            self.handle_exception("creating {}".format(entity_name), result)

    def send_put(self, url, entity_name, body=None):
        agent = self.resolve_user_agent()
        query_params = self.build_query_params()
        headers = dict(
            {
                'User-Agent': agent,
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + self.auth_token
            }
        )

        self.print_output("Sending put request to spotinst API.")
        result = requests.put(
            url,
            params=query_params,
            data=body,
            headers=headers)

        if result.status_code == requests.codes.ok:
            self.print_output("Success")
            data = json.loads(result.content.decode('utf-8'))
            return data
        else:
            self.handle_exception("updating {}".format(entity_name), result)

    def send_put_with_params(self, body, url, entity_name, user_query_params):
        agent = self.resolve_user_agent()
        query_params = self.build_query_params_with_input(user_query_params)

        headers = dict(
            {
                'User-Agent': agent,
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + self.auth_token
            }
        )

        self.print_output("Sending put request to spotinst API.")
        result = requests.put(
            url,
            params=query_params,
            data=body,
            headers=headers)

        if result.status_code == requests.codes.ok:
            self.print_output("Success")
            data = json.loads(result.content.decode('utf-8'))
            return data
        else:
            self.handle_exception("updating {}".format(entity_name), result)

    def resolve_user_agent(self):
        global _SpotinstClient__spotinst_sdk_user_agent
        agent = _SpotinstClient__spotinst_sdk_user_agent
        if self.user_agent is not None:
            agent = '{}+{}'.format(self.user_agent, agent)
        return agent

    def handle_exception(self, action_string, result):
        self.print_output(result.status_code)
        data = json.loads(result.content.decode('utf-8'))
        response_json = json.dumps(data["response"])
        self.print_output(response_json)
        raise SpotinstClientException(
            "Error encountered while " +
            action_string,
            response_json)

    def convert_json(self, val, convert):
        new_json = {}
        if val is None:
            return val
        elif type(val) in (int, float, bool, "".__class__, u"".__class__):
            return val

        for k, v in list(val.items()):
            new_v = v
            if isinstance(v, dict):
                new_v = self.convert_json(v, convert)
            elif isinstance(v, list):
                new_v = list()
                for x in v:
                    new_v.append(self.convert_json(x, convert))
            new_json[convert(k)] = new_v
        return new_json

    def exclude_missing(self, obj):
        # Delete keys with the value 'none' in a dictionary, recursively.

        # if obj.items() is not None:
        if obj.items() is not None:
            for key, value in list(obj.items()):

                # Remove none values
                if value == aws_elastigroup.none:
                    del obj[key]

                # Handle Objects
                elif isinstance(value, dict):
                    self.exclude_missing(obj=value)

                # Handle lists
                elif self.is_sequence(arg=value):
                    for listitem in value:
                        # Handle Lists of objects
                        try:
                            self.exclude_missing(obj=listitem)
                        except AttributeError:
                            pass
        return obj  # For convenience

    def is_sequence(self, arg):
        return (not hasattr(arg, "strip") and
                hasattr(arg, "__getitem__") or
                hasattr(arg, "__iter__"))

    def build_query_params(self):
        query_params = None
        if self.account_id is not None:
            query_params = dict({self.__account_id_key: self.account_id})

        return query_params

    def build_query_params_with_input(self, user_params):
        query_params = dict()
        if self.account_id is not None:
            query_params = dict({self.__account_id_key: self.account_id})

        if user_params is not None:
            query_params = self.merge_two_dicts(query_params, user_params)

        return query_params

    @staticmethod
    def merge_two_dicts(x, y):
        z = x.copy()  # start with x's keys and values
        z.update(y)  # modifies z with y's keys and values & returns None
        return z

    def camel_to_underscore(self, name):
        return self.camel_pat.sub(lambda x: '_' + x.group(1).lower(), name)

    def underscore_to_camel(self, name):
        return self.under_pat.sub(lambda x: x.group(1).upper(), name)

    def load_credentials(self, profile, credentials_file):
        self.account_id = os.environ.get(VAR_SPOTINST_ACCOUNT, None)
        self.auth_token = os.environ.get(VAR_SPOTINST_TOKEN, None)

        if not self.auth_token:
            if not profile:
                profile = os.environ.get(VAR_SPOTINST_PROFILE, DEFAULT_PROFILE)

            if not credentials_file:
                credentials_file = os.environ.get(
                    VAR_SPOTINST_SHARED_CREDENTIALS_FILE,
                    DEFAULT_CREDENTIALS_FILE)

            with open(credentials_file, 'r') as credentials_file:
                config = yaml.load(credentials_file)

                if config:
                    self.account_id = config.get(
                        profile, {}).get(
                        "account", None)
                    self.auth_token = config.get(
                        profile, {}).get("token", None)

            if not self.auth_token:
                raise SpotinstClientException("failed to load credentials")

    # endregion


class SpotinstClientException(Exception):
    def __init__(self, message, response):
        message = message + "\n" + response
        # Call the base class constructor with the parameters it needs
        super(SpotinstClientException, self).__init__(message)



import json

none = "d3043820717d74d9a17694c176d39733"

# region EMR
class EMR:

	def __init__(
		self,
		name=none,
		description=none,
		region=none,
		strategy=none,
		compute=none,
		scaling=none):
		"""

		:type name: str
		:type decription: str
		:type region: str
		:type strategy: Strategy
		:tpye compute: Compute
		:type scaling: Scaling
		"""
		self.name = name
		self.description = description
		self.region = region
		self.strategy = strategy
		self.compute = compute
		self.scaling = scaling

# endregion

# region Strategy
class Strategy:

	def __init__(
		self,
		wrapping=none,
		cloning=none,
		provisioning_timeout=none):
		"""

		:type wrapping: Wrapping
		:type cloning: Cloning
		:type provisioning_timeout: ProvisioningTimeout
		"""
		self.wrapping = wrapping
		self.cloning = cloning
		self.provisioning_timeout = provisioning_timeout

class Wrapping:

	def __init__(
		self,
		source_cluster_id):
		"""

		:type source_cluster_id: str
		"""
		self.source_cluster_id = source_cluster_id


class Cloning:

	def __init__(
		self,
		origin_cluster_id=none,
		include_steps=none):
		"""

		:type origin_cluster_id: str
		:type include_steps: bool
		"""
		self.origin_cluster_id = origin_cluster_id
		self.include_steps = include_steps


class ProvisioningTimeout:

	def __init__(
		self, 
		timeout = none,
		timeout_action = none):
		"""

		:type timeout: int
		:tpye timeout_action: str
		"""
		self.timeout = timeout
		self.timeout_action = timeout_action


# endregion

# region Compute
class Compute:

	def __init__(
		self,
		ebs_root_volume_size=none,
		availability_zones=none,
		bootstrap_actions=none,
		steps=none,
		instance_groups=none,
		configurations=none):
		"""

		:type ebs_root_volume_size: int
		:type availability_zones: List[AvailabilityZone]
		:type bootstrap_actions: BootstrapActions
		:type steps: Steps
		:type instance_groups: InstanceGroups
		:type configurations: Configurations
		"""
		self.ebs_root_volume_size = ebs_root_volume_size
		self.availability_zones = availability_zones
		self.bootstrap_actions = bootstrap_actions
		self.steps = steps
		self.instance_groups = instance_groups
		self.configurations = configurations


class AvailabilityZone:

	def __init__(
		self,
		name=none,
		subnet=none):
		"""

		:type name: str
		:type subnet: str
		"""
		self.name = name
		self.subnet = subnet


class BootstrapActions:

	def __init__(
		self,
		file=none):
		"""

		:type file: File
		"""
		self.file = file


class File:

	def __init__(
		self, 
		bucket=none,
		key=none):
		"""

		:type bucket: str
		:type key: str
		"""
		self.bucket = bucket
		self. key = key


class Steps:

	def __init__(
		self,
		file=none):
		"""
		:type file: File
		"""
		self.file = file


class InstanceGroups:

	def __init__(
		self, 
		master_group=none,
		core_group=none,
		task_group=none):
		"""

		:type master_group: MasterGroup
		:type core_group: CoreGroup
		:type task_group: TaskGroup
		"""
		self.master_group = master_group
		self.core_group = core_group
		self.task_group = task_group


class MasterGroup:

	def __init__(
		self,
		instance_types=none,
		target=none,
		life_cycle=none):
		"""

		:type instance_types: List[str]
		:type target: int
		:type life_cycle: str
		"""
		self.instance_types = instance_types
		self.target = target
		self.life_cycle = life_cycle


class CoreGroup:

	def __init__(
		self,
		instance_types=none,
		target=none,
		life_cycle=none,
		ebs_configuration=none):
		"""

		:type instance_types: List[str]
		:type target: int
		:type life_cycle: str
		:type ebs_configuration: EbsConfiguration
		"""
		self.instance_types = instance_types
		self.target = target
		self.life_cycle = life_cycle
		self.ebs_configuration = ebs_configuration


class TaskGroup:

	def __init__(
		self,
		instance_types=none,
		capacity=none,
		life_cycle=none,
		ebs_configuration=none):
		"""

		:type instance_types: List[str]
		:type capacity: Capacity
		:type life_cycle: str
		:type ebs_configuration: EbsConfiguration
		"""
		self.instance_types = instance_types
		self.capacity = capacity
		self.life_cycle = life_cycle
		self.ebs_configuration = ebs_configuration


class Capacity:

	def __init__(
		self,
		target=none,
		minimum=none,
		maximum=none):
		"""
		
		:type target: int
		:type minimum: int
		:type maximum: int
		"""
		self.target = target
		self.minimum = minimum
		self.maximum = maximum


class EbsConfiguration:

	def __init__(
		self,
		ebs_block_device_configs=none,
		ebs_optimized=none):
		"""

		:type ebs_block_device_configs: List[SingleEbsConfig]
		:type ebs_optimized: bool
		"""
		self.ebs_block_device_configs = ebs_block_device_configs
		self.ebs_optimized = ebs_optimized


class SingleEbsConfig:

	def __init__(
		self,
		volume_specification=none,
		volumes_per_instance=none):
		"""

		:type volume_specification: VolumeSpecification
		:type volumes_per_instance: int
		"""
		self.volume_specification = volume_specification
		self.volumes_per_instance = volumes_per_instance


class VolumeSpecification:
	def __init__(
		self,
		volume_type=none,
		size_in_gb=none):
		"""

		:type volume_type: str
		:type size_in_GB: int
		"""
		self.volume_type = volume_type
		self.size_in_gB = size_in_gb


class Configurations:

	def __init__(
		self,
		file=none):
		"""

		:type file: File
		"""
		self.file = file


# endregion

# region Scaling
class Scaling:

	def __init__(
		self,
		up=none,
		down=none):
		"""

		:type up: List[Metric]
		:type down: List[Metric]
		"""
		self.up = up
		self.down = down


class Metric:

	def __init__(
		self,
		metric_name=none,
		statistic=none,
		unit=none,
		threshold=none,
		adjustment=none,
		namespace=none,
		period=none,
		evaluation_periods=none,
		action=none,
		cooldown=none,
		dimensions=none,
		operator=none):
		"""

		:type metric_name: str
		:type statistic: str
		:type unit: str
		:type threshold: int
		:type adjustment: int
		:type namespace: str
		:type period: int
		:type evaluation_periods: int
		:type action: Action
		:type cooldown: int
		:type dimensions: List[Dimension]
		:type operator: str
		"""
		self.metric_name = metric_name
		self.statistic = statistic
		self.unit = unit
		self.threshold = threshold
		self.adjustment = adjustment
		self.namespace = namespace
		self.period = period
		self.evaluation_periods = evaluation_periods
		self.action = action
		self.cooldown = cooldown
		self.dimensions = dimensions
		self.operator = operator

class Action:

	def __init__(
		self,
		type=none,
		adjustment=none,
		min_target_capacity=none,
		target=none,
		minimum=none,
		maximum=none):
		"""

		:type type: str
		:type adjustment: int
		:type min_target_capacity: int
		:type target: int
		:type minimum: int
		:type maximum: int
		"""
		self.type = type
		self.adjustment = adjustment
		self.min_target_capacity = min_target_capacity
		self.target = target
		self.minimum = minimum
		self.maximum = maximum

class Dimension:

	def __init__(
		self,
		name=none):
		"""
		:type name: str
		"""
		self.name = name


#endregion



class EMRCreationRequest:
    def __init__(self, mrScaler):
        self.mrScaler = mrScaler

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)










import base64
import json
import os
import tempfile
import zipfile

none = "d3043820717d74d9a17694c176d39733"


# region Application
class Application:
    def __init__(self, name):
        self.name = name


# endregion

# region Environment
class Environment:
    def __init__(self, name, application_id, providers=none, locations=none):
        self.name = name
        self.application_id = application_id
        self.preferences = {'providers': providers, 'locations': locations}


# endregion

# region Environment
class Function:
    def __init__(
            self,
            name,
            environment_id,
            directory,
            handler,
            runtime,
            memory,
            timeout):
        self.name = name
        self.environment_id = environment_id
        self.directory = directory
        self.handler = handler
        self.runtime = runtime
        self.memory = memory
        self.timeout = timeout


# endregion


class ApplicationCreationRequest:
    def __init__(self, application):
        self.application = application

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class EnvironmentCreationRequest:
    def __init__(self, environment):
        self.environment = environment

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class FunctionCreationRequest:
    def __init__(self, function, print_output=True):
        self.should_print_output = print_output
        self.function = self.rebuildFunctionInlineCode(function)

    def print_output(self, output):
        if self.should_print_output is True:
            print(output)

    def rebuildFunctionInlineCode(self, function):
        directory = function.directory
        handler = function.handler

        with tempfile.NamedTemporaryFile() as temp:
            self.zip(directory, temp.name)

            temp.seek(0)
            base64content = base64.b64encode(temp.read())
            function.code = {'source': base64content, 'handler': handler}
            del function.directory
            del function.handler

        return function

    def zip(self, src, dst):
        zf = zipfile.ZipFile(dst, "w", zipfile.ZIP_DEFLATED)
        abs_src = os.path.abspath(src)
        for dirname, subdirs, files in os.walk(src):
            for filename in files:
                absname = os.path.abspath(os.path.join(dirname, filename))
                arcname = absname[len(abs_src) + 1:]
                self.print_output(
                    "collecting file {}".format(
                        os.path.join(
                            dirname, filename)))
                zf.write(absname, arcname)
        zf.close()

    def toJSON(self):
        del self.should_print_output
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)



# Always prefer setuptools over distutils
from codecs import open
from os import path

import sys

from setuptools import setup

needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
pytest_runner = ['pytest-runner'] if needs_pytest else []

here = path.abspath(path.dirname(__file__))

# Get the long description from the README.md file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

version = {}
with open("spotinst_sdk/version.py") as fp:
    exec(fp.read(), version)

setup(
    name='spotinst-sdk',

    version=version['__version__'],

    description='A Python SDK for Spotinst',
    long_description=long_description,
    long_description_content_type='text/markdown',

    # The project's main homepage.
    url='https://github.com/spotinst/spotinst-sdk-python',

    # Author details
    author='Spotinst',
    author_email='service@spotinst.com',

    # Choose your license
    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6'
    ],

    keywords='spotinst spot instances aws ec2 cloud infrastructure development elastigroup',
    packages=["spotinst_sdk"],
    install_requires=['requests', 'PyYaml'],

    setup_requires=[] + pytest_runner,
    tests_require=["pytest"]
)



import json

none = "d3043820717d74d9a17694c176d39733"

# region StatefulInstance
class StatefulInstance:

	def __init__(
		self,
		should_keep_private_ip=none,
		original_instance_id=none,
		name=none,
		product=none,
		spot_instance_types=none,
		region=none,
		availability_zones=none):
		"""

		:type shouldKeepPrivateIp: bool
		"type originalInstanceId: str
		"type name: str
		:type product: str
		:type spotInstanceTypes: List[str]
		:type region: str
		:type availabilityZones: List[AvailabiltyZones]
		"""
		self.should_keep_private_ip = should_keep_private_ip
		self.original_instance_id = original_instance_id
		self.name = name
		self.product = product
		self.spot_instance_types = spot_instance_types
		self.region = region
		self.availability_zones = availability_zones

class AvailabilityZone:

	def __init__(
		self,
		name=none,
		subnet_ids=none):
		"""

		:type name: str
		:type subnet: str
		"""
		self.name = name
		self.subnet_ids = subnet_ids

# endregion


class StatefulImportRequest:
    def __init__(self, statefulMigrationGroup):
        self.statefulMigrationGroup = statefulMigrationGroup

        print(self.statefulMigrationGroup)

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)






import json

none = "d3043820717d74d9a17694c176d39733"


# region Elastigroup
class Elastigroup:

    def __init__(
            self,
            name=none,
            description=none,
            region=none,
            capacity=none,
            strategy=none,
            compute=none,
            scaling=none,
            scheduling=none,
            multai=none,
            third_parties_integration=none):
        """

        :type name: str
        :type description: str
        :type region: str
        :type capacity: Capacity
        :type strategy: Strategy
        :type compute: Compute
        :type scaling: Scaling
        :type scheduling: Scheduling
        :type multai: Multai
        :type third_parties_integration: ThirdPartyIntegrations
        """
        self.name = name
        self.description = description
        self.region = region
        self.capacity = capacity
        self.strategy = strategy
        self.scaling = scaling
        self.scheduling = scheduling
        self.multai = multai
        self.third_parties_integration = third_parties_integration
        self.compute = compute


# endregion

# region Strategy
class Strategy:

    def __init__(
            self,
            availability_vs_cost=none,
            risk=none,
            utilize_reserved_instances=none,
            fallback_to_od=none,
            on_demand_count=none,
            draining_timeout=none,
            spin_up_time=none,
            lifetime_period=none,
            signals=none,
            scaling_strategy=none,
            persistence=none,
            revert_to_spot=none):
        """

        :type availability_vs_cost: str
        :type risk: int
        :type utilize_reserved_instances: bool
        :type fallback_to_od: bool
        :type on_demand_count: int
        :type draining_timeout: int
        :type spin_up_time: int
        :type lifetime_period: int
        :type signals: list[Signal]
        :type scaling_strategy: ScalingStrategy
        :type persistence: Persistence
        :type revert_to_spot: RevertToSpot
        """
        self.risk = risk
        self.utilize_reserved_instances = utilize_reserved_instances
        self.fallback_to_od = fallback_to_od
        self.on_demand_count = on_demand_count
        self.availability_vs_cost = availability_vs_cost
        self.draining_timeout = draining_timeout
        self.spin_up_time = spin_up_time
        self.lifetime_period = lifetime_period
        self.spin_up_time = spin_up_time
        self.signals = signals
        self.scaling_strategy = scaling_strategy
        self.persistence = persistence
        self.revert_to_spot = revert_to_spot


class Signal:

    def __init__(self, name=none, timeout=none):
        """

        :type name: str
        :type timeout: int
        """
        self.name = name
        self.timeout = timeout


class ScalingStrategy:
    def __init__(self, terminate_at_end_of_billing_hour):
        """

        :type terminate_at_end_of_billing_hour: bool
        """
        self.terminate_at_end_of_billing_hour = terminate_at_end_of_billing_hour


class Persistence:
    def __init__(
            self,
            should_persist_block_devices=none,
            should_persist_root_device=none,
            should_persist_private_ip=none,
            block_devices_mode=none):
        """

        :type should_persist_block_devices: bool
        :type should_persist_root_device: bool
        :type should_persist_private_ip: bool
        :type block_devices_mode: str
        """
        self.should_persist_block_devices = should_persist_block_devices
        self.should_persist_root_device = should_persist_root_device
        self.should_persist_private_ip = should_persist_private_ip
        self.block_devices_mode = block_devices_mode


class RevertToSpot:
    def __init__(self, perform_at=none, time_windows=none):
        """

        :type perform_at: str
        :type time_windows: list[str]
        """
        self.perform_at = perform_at
        self.time_windows = time_windows


# endregion

# region Capacity
class Capacity:
    def __init__(self, minimum=none, maximum=none, target=none, unit=none):
        """

        :type minimum: int
        :type maximum: int
        :type target: int
        :type unit: str
        """
        self.minimum = minimum
        self.maximum = maximum
        self.target = target
        self.unit = unit


# endregion

# region Scaling
class Scaling:
    def __init__(self, up=none, down=none, target=none):
        """

        :type up:  list[ScalingPolicy]
        :type down: list[ScalingPolicy]
        :type target: list[TargetTrackingPolicy]
        """
        self.up = up
        self.down = down
        self.target = target


class ScalingPolicyDimension:
    def __init__(self, name=none, value=none):
        """

        :type name: str
        :type value: str
        """
        self.name = name
        self.value = value


class ScalingPolicyAction:
    def __init__(self, type=none, adjustment=none, min_target_capacity=none,
                 max_target_capacity=none, target=none,
                 minimum=none,
                 maximum=none):
        """

        :type type: str
        :type adjustment: int
        :type min_target_capacity: int
        :type max_target_capacity: int
        :type target: int
        :type minimum: int
        :type maximum: int
        """
        self.type = type
        self.adjustment = adjustment
        self.min_target_capacity = min_target_capacity
        self.max_target_capacity = max_target_capacity
        self.target = target
        self.minimum = minimum
        self.maximum = maximum


class ScalingPolicy:
    def __init__(
            self,
            namespace=none,
            metric_name=none,
            statistic=none,
            evaluation_periods=none,
            period=none,
            threshold=none,
            cooldown=none,
            action=none,
            unit=none,
            operator=none,
            dimensions=none,
            policy_name=none,
            source=none,
            extended_statistic=none):
        """

        :type namespace: str
        :type metric_name: str
        :type statistic: str
        :type evaluation_periods: int
        :type period: int
        :type threshold: float
        :type cooldown: int
        :type action: ScalingPolicyAction
        :type unit: str
        :type operator: str
        :type dimensions: list[ScalingPolicyDimension]
        :type policy_name: str
        :type source: str
        :type extended_statistic: str
        """
        self.policy_name = policy_name
        self.namespace = namespace
        self.metric_name = metric_name
        self.dimensions = dimensions
        self.statistic = statistic
        self.evaluation_periods = evaluation_periods
        self.period = period
        self.threshold = threshold
        self.cooldown = cooldown
        self.action = action
        self.unit = unit
        self.operator = operator
        self.source = source
        self.extended_statistic = extended_statistic


class TargetTrackingPolicy:
    def __init__(self, namespace=none, metric_name=none, statistic=none,
                 cooldown=none, target=none, unit=none,
                 dimensions=none, policy_name=none, source=none):
        """

        :type namespace: str
        :type metric_name: str
        :type statistic: str
        :type cooldown: int
        :type target: int
        :type unit: str
        :type dimensions: list[ScalingPolicyDimension]
        :type policy_name: str
        :type source: str
        """
        self.policy_name = policy_name
        self.namespace = namespace
        self.source = source
        self.metric_name = metric_name
        self.dimensions = dimensions
        self.statistic = statistic
        self.unit = unit
        self.cooldown = cooldown
        self.target = target


# endregion

# region Scheduling
class Scheduling:
    def __init__(self, tasks=none):
        """

        :type tasks: list[ScheduledTask]
        """
        self.tasks = tasks


class ScheduledTask:
    def __init__(
            self,
            task_type=none,
            scale_target_capacity=none,
            scale_min_capacity=none,
            scale_max_capacity=none,
            target_capacity=none,
            min_capacity=none,
            max_capacity=none,
            batch_size_percentage=none,
            grace_period=none,
            adjustment=none,
            adjustment_percentage=none,
            is_enabled=none,
            frequency=none,
            cron_expression=none,
            start_time=none):
        """

        :type task_type: str
        :type scale_target_capacity: int
        :type scale_min_capacity: int
        :type scale_max_capacity: int
        :type target_capacity: int
        :type min_capacity: int
        :type max_capacity: int
        :type batch_size_percentage: int
        :type grace_period: int
        :type adjustment: int
        :type adjustment_percentage: int
        :type is_enabled: bool
        :type frequency: str
        :type cron_expression: str
        :type start_time: str
        """
        self.is_enabled = is_enabled
        self.frequency = frequency
        self.cron_expression = cron_expression
        self.task_type = task_type
        self.scale_target_capacity = scale_target_capacity
        self.scale_min_capacity = scale_min_capacity
        self.scale_max_capacity = scale_max_capacity
        self.target_capacity = target_capacity
        self.min_capacity = min_capacity
        self.max_capacity = max_capacity
        self.batch_size_percentage = batch_size_percentage
        self.grace_period = grace_period
        self.adjustment = adjustment
        self.adjustment_percentage = adjustment_percentage
        self.start_time = start_time


# endregion

# region Multai
class Multai:
    def __init__(self, token=none, balancers=none):
        """

        :type token: str
        :type balancers: str
        """
        self.token = token
        self.balancers = balancers


class MultaiLoadBalancer:
    def __init__(
            self,
            project_id=none,
            balancer_id=none,
            target_set_id=none,
            az_awareness=none,
            auto_weight=none):
        """

        :type project_id: str
        :type balancer_id: str
        :type target_set_id: str
        :type az_awareness: bool
        :type auto_weight: bool
        """
        self.project_id = project_id
        self.balancer_id = balancer_id
        self.target_set_id = target_set_id
        self.az_awareness = az_awareness
        self.auto_weight = auto_weight


# endregion

# region ThirdPartyIntegrations
class Rancher:

    def __init__(self, access_key=none, secret_key=none, master_host=none, version=none):
        """

        :type access_key: str
        :type secret_key: str
        :type master_host: str
        :type version: str
        """
        self.access_key = access_key
        self.secret_key = secret_key
        self.master_host = master_host
        self.version = version


class Mesosphere:
    def __init__(self, api_server=none):
        """

        :type api_server: str
        """
        self.api_server = api_server


class ElasticBeanstalk:
    def __init__(self, environment_id=none, deployment_preferences=none):
        """

        :type environment_id: str
        :type deployment_preferences: DeploymentPreferences
        """
        self.environment_id = environment_id
        self.deployment_preferences = deployment_preferences


class DeploymentPreferences:
    def __init__(
            self,
            automatic_roll=none,
            batch_size_percentage=none,
            grace_period=none,
            strategy=none):
        """

        :type automatic_roll: bool
        :type batch_size_percentage: int
        :type grace_period: int
        :type strategy: BeanstalkDeploymentStrategy
        """
        self.automatic_roll = automatic_roll
        self.batch_size_percentage = batch_size_percentage
        self.grace_period = grace_period
        self.strategy = strategy


class BeanstalkDeploymentStrategy:
    def __init__(self, action=none, should_drain_instances=none):
        """

        :type action: str
        :type should_drain_instances: bool
        """
        self.action = action
        self.should_drain_instances = should_drain_instances


class EcsConfiguration:
    def __init__(self, cluster_name=none, auto_scale=none):
        """

        :type cluster_name: str
        :type auto_scale: EcsAutoScaleConfiguration
        """
        self.cluster_name = cluster_name
        self.auto_scale = auto_scale


class EcsAutoScaleConfiguration:
    def __init__(
            self,
            is_enabled=none,
            is_auto_config=none,
            cooldown=none,
            headroom=none,
            attributes=none,
            down=none):
        """

        :type is_enabled: bool
        :type is_auto_config:  bool
        :type cooldown: int
        :type headroom: EcsAutoScalerHeadroomConfiguration
        :type attributes: list[EcsAutoScalerAttributeConfiguration]
        :type down: EcsAutoScalerDownConfiguration
        """
        self.is_enabled = is_enabled
        self.is_auto_config = is_auto_config
        self.cooldown = cooldown
        self.headroom = headroom
        self.attributes = attributes
        self.down = down


class EcsAutoScalerHeadroomConfiguration:
    def __init__(
            self,
            cpu_per_unit=none,
            memory_per_unit=none,
            num_of_units=none):
        """

        :type cpu_per_unit: int
        :type memory_per_unit: int
        :type num_of_units: int
        """
        self.cpu_per_unit = cpu_per_unit
        self.memory_per_unit = memory_per_unit
        self.num_of_units = num_of_units


class EcsAutoScalerAttributeConfiguration:
    def __init__(self, key=none, value=none):
        """

        :type key: str
        :type value: str
        """
        self.key = key
        self.value = value


class EcsAutoScalerDownConfiguration:
    def __init__(self, evaluation_periods=none):
        """

        :type evaluation_periods: int
        """
        self.evaluation_periods = evaluation_periods


class MlbRuntimeConfiguration:
    def __init__(self, deployment_id=none):
        """

        :type deployment_id: str
        """
        self.deployment_id = deployment_id


class KubernetesConfiguration:
    def __init__(
            self,
            api_server=none,
            token=none,
            integration_mode=none,
            cluster_identifier=none,
            auto_scale=none):
        """

        :type api_server: str
        :type token: str
        :type integration_mode: str
        :type cluster_identifier: str
        :type auto_scale: KubernetesAutoScalerConfiguration
        """
        self.api_server = api_server
        self.token = token
        self.integration_mode = integration_mode
        self.cluster_identifier = cluster_identifier
        self.auto_scale = auto_scale


class KubernetesAutoScalerConfiguration:
    def __init__(
            self,
            is_enabled=none,
            is_auto_config=none,
            cooldown=none,
            headroom=none,
            labels=none,
            down=none):
        """

        :type is_enabled: bool
        :type is_auto_config: bool
        :type cooldown: int
        :type headroom: KubernetesAutoScalerHeadroomConfiguration
        :type labels: KubernetesAutoScalerLabelsConfiguration
        :type down: KubernetesAutoScalerDownConfiguration
        """
        self.is_enabled = is_enabled
        self.is_auto_config = is_auto_config
        self.cooldown = cooldown
        self.headroom = headroom
        self.labels = labels
        self.down = down


class KubernetesAutoScalerHeadroomConfiguration:
    def __init__(
            self,
            cpu_per_unit=none,
            memory_per_unit=none,
            num_of_units=none):
        """

        :type cpu_per_unit: int
        :type memory_per_unit: int
        :type num_of_units: int
        """
        self.cpu_per_unit = cpu_per_unit
        self.memory_per_unit = memory_per_unit
        self.num_of_units = num_of_units


class KubernetesAutoScalerLabelsConfiguration:
    def __init__(self, key=none, value=none):
        """

        :type key: str
        :type value: str
        """
        self.key = key
        self.value = value


class KubernetesAutoScalerDownConfiguration:
    def __init__(self, evaluation_periods=none):
        """

        :type evaluation_periods: int
        """
        self.evaluation_periods = evaluation_periods


class RightScaleConfiguration:
    def __init__(self, account_id=none, refresh_token=none, region=none):
        """

        :type account_id: str
        :type refresh_token: str
        :type region: str
        """
        self.account_id = account_id
        self.refresh_token = refresh_token
        self.region = region


class OpsWorksConfiguration:
    def __init__(self, layer_id=none, stack_type=none):
        """

        :type layer_id: str
        :type stack_type: str
        """
        self.layer_id = layer_id
        self.stack_type = stack_type


class ChefConfiguration:
    def __init__(
            self,
            chef_server=none,
            organization=none,
            user=none,
            pem_key=none,
            chef_version=none):
        """

        :type chef_server: str
        :type organization: str
        :type user: str
        :type pem_key: str
        :type chef_version: str
        """
        self.chef_server = chef_server
        self.organization = organization
        self.user = user
        self.pem_key = pem_key
        self.chef_version = chef_version


class CodeDeployConfiguration:
    def __init__(
            self,
            deployment_groups=none,
            clean_up_on_failure=none,
            terminate_instance_on_failure=none):
        """

        :type deployment_groups: list[CodeDeployDeploymentGroupsConfiguration]
        :type clean_up_on_failure: bool
        :type terminate_instance_on_failure: bool
        """
        self.deployment_groups = deployment_groups
        self.clean_up_on_failure = clean_up_on_failure
        self.terminate_instance_on_failure = terminate_instance_on_failure


class CodeDeployDeploymentGroupsConfiguration:
    def __init__(self, application_name=none, deployment_group_name=none):
        """

        :type application_name: str
        :type deployment_group_name: str
        """
        self.application_name = application_name
        self.deployment_group_name = deployment_group_name


class NomadConfiguration:
    def __init__(
            self,
            master_host=none,
            master_port=none,
            acl_token=none,
            auto_scale=none):
        """

        :type master_host: str
        :type master_port: int
        :type acl_token: str
        :type auto_scale: NomadAutoScalerConfiguration
        """
        self.master_host = master_host
        self.master_port = master_port
        self.acl_token = acl_token
        self.auto_scale = auto_scale


class NomadAutoScalerConfiguration:
    def __init__(
            self,
            is_enabled=none,
            cooldown=none,
            headroom=none,
            constraints=none,
            down=none):
        """

        :type is_enabled: bool
        :type cooldown: int
        :type headroom: NomadAutoScalerHeadroomConfiguration
        :type constraints: list[NomadAutoScalerConstraintsConfiguration]
        :type down: NomadAutoScalerDownConfiguration
        """
        self.is_enabled = is_enabled
        self.cooldown = cooldown
        self.headroom = headroom
        self.constraints = constraints
        self.down = down


class NomadAutoScalerHeadroomConfiguration:
    def __init__(
            self,
            cpu_per_unit=none,
            memory_per_unit=none,
            num_of_units=none):
        """

        :type cpu_per_unit: int
        :type memory_per_unit: int
        :type num_of_units: int
        """
        self.cpu_per_unit = cpu_per_unit
        self.memory_per_unit = memory_per_unit
        self.num_of_units = num_of_units


class NomadAutoScalerConstraintsConfiguration:
    def __init__(self, key=none, value=none):
        """

        :type key: str
        :type value: str
        """
        self.key = key
        self.value = value


class NomadAutoScalerDownConfiguration:
    def __init__(self, evaluation_periods=none):
        """

        :type evaluation_periods: int
        """
        self.evaluation_periods = evaluation_periods


class DockerSwarmConfiguration:
    def __init__(self, master_host=none, master_port=none, auto_scale=none):
        """

        :type master_host: str
        :type master_port: int
        :type auto_scale: DockerSwarmAutoScalerConfiguration
        """
        self.master_host = master_host
        self.master_port = master_port
        self.auto_scale = auto_scale


class DockerSwarmAutoScalerConfiguration:
    def __init__(
            self,
            is_enabled=none,
            cooldown=none,
            headroom=none,
            down=none):
        """

        :type is_enabled: bool
        :type cooldown: int
        :type headroom: DockerSwarmAutoScalerHeadroomConfiguration
        :type down: DockerSwarmAutoScalerDownConfiguration
        """
        self.is_enabled = is_enabled
        self.cooldown = cooldown
        self.headroom = headroom
        self.down = down


class DockerSwarmAutoScalerHeadroomConfiguration:
    def __init__(
            self,
            cpu_per_unit=none,
            memory_per_unit=none,
            num_of_units=none):
        """

        :type cpu_per_unit: int
        :type memory_per_unit:
        :type num_of_units: int
        """
        self.cpu_per_unit = cpu_per_unit
        self.memory_per_unit = memory_per_unit
        self.num_of_units = num_of_units


class DockerSwarmAutoScalerDownConfiguration:
    def __init__(self, evaluation_periods=none):
        """

        :type evaluation_periods: int
        """
        self.evaluation_periods = evaluation_periods


class Route53Configuration:
    def __init__(self, domains=none):
        """

        :type domains: list[Route53DomainsConfiguration]
        """
        self.domains = domains


class Route53DomainsConfiguration:
    def __init__(self, hosted_zone_id=none, record_sets=none):
        """

        :type hosted_zone_id: str
        :type record_sets: list[Route53RecordSetsConfiguration]
        """
        self.hosted_zone_id = hosted_zone_id
        self.record_sets = record_sets


class Route53RecordSetsConfiguration:
    def __init__(self, name=none, use_public_ip=none):
        """

        :type name: str
        :type use_public_ip: bool
        """
        self.name = name
        self.use_public_ip = use_public_ip


class ThirdPartyIntegrations:
    def __init__(
            self,
            rancher=none,
            mesosphere=none,
            elastic_beanstalk=none,
            ecs=none,
            kubernetes=none,
            right_scale=none,
            ops_works=none,
            chef=none,
            mlb_runtime=none,
            code_deploy=none,
            nomad=none,
            docker_swarm=none,
            route53=none):
        """

        :type rancher: Rancher
        :type mesosphere: Mesosphere
        :type elastic_beanstalk: ElasticBeanstalk
        :type ecs: EcsConfiguration
        :type kubernetes: KubernetesConfiguration
        :type right_scale: RightScaleConfiguration
        :type ops_works: OpsWorksConfiguration
        :type chef: ChefConfiguration
        :type mlb_runtime: MlbRuntimeConfiguration
        :type code_deploy: CodeDeployConfiguration
        :type nomad: NomadConfiguration
        :type docker_swarm: DockerSwarmConfiguration
        :type route53: Route53Configuration
        """
        self.rancher = rancher
        self.mesosphere = mesosphere
        self.elastic_beanstalk = elastic_beanstalk
        self.ecs = ecs
        self.kubernetes = kubernetes
        self.right_scale = right_scale
        self.ops_works = ops_works
        self.chef = chef
        self.mlb_runtime = mlb_runtime
        self.code_deploy = code_deploy
        self.nomad = nomad
        self.docker_swarm = docker_swarm
        self.route53 = route53


# endregion

# region Compute
class Compute:
    def __init__(
            self,
            launch_specification=none,
            instance_types=none,
            product=none,
            availability_zones=none,
            elastic_ips=none,
            private_ips=none,
            subnet_ids=none,
            preferred_availability_zones=none):
        """

        :type launch_specification: LaunchSpecification
        :type instance_types: InstanceTypes
        :type product: str
        :type availability_zones: list[AvailabilityZone]
        :type elastic_ips: list[str]
        :type private_ips: list[str]
        :type subnet_ids: list[str]
        :type preferred_availability_zones: list[str]
        """
        self.elastic_ips = elastic_ips
        self.private_ips = private_ips
        self.subnet_ids = subnet_ids
        self.instance_types = instance_types
        self.availability_zones = availability_zones
        self.product = product
        self.launch_specification = launch_specification
        self.preferred_availability_zones = preferred_availability_zones


class AvailabilityZone:
    def __init__(
            self,
            name=none,
            subnet_id=none,
            subnet_ids=none,
            placement_group_name=none):
        """

        :type name:
        :type subnet_id:
        :type subnet_ids:
        :type placement_group_name:
        """
        self.name = name
        self.subnet_id = subnet_id
        self.subnet_ids = subnet_ids
        self.placement_group_name = placement_group_name


class InstanceTypes:
    def __init__(
            self,
            ondemand=none,
            spot=none,
            weights=none,
            preferred_spot=none):
        """

        :type ondemand: str
        :type spot: list[str]
        :type weights: list[Weight]
        :type preferred_spot: list[str]
        """
        self.ondemand = ondemand
        self.spot = spot
        self.weights = weights
        self.preferred_spot = preferred_spot


class Weight:
    def __init__(self, instance_type=none, weighted_capacity=none):
        """

        :type instance_type: str
        :type weighted_capacity: int
        """
        self.instance_type = instance_type
        self.weighted_capacity = weighted_capacity


class LaunchSpecification:
    def __init__(
            self,
            security_group_ids=none,
            image_id=none,
            monitoring=none,
            health_check_type=none,
            load_balancers_config=none,
            health_check_grace_period=none,
            health_check_unhealthy_duration_before_replacement=none,
            ebs_optimized=none,
            tenancy=none,
            iam_role=none,
            key_pair=none,
            user_data=none,
            shutdown_script=none,
            block_device_mappings=none,
            network_interfaces=none,
            tags=none):
        """

        :type security_group_ids: list[str]
        :type image_id: str
        :type monitoring: bool
        :type health_check_type: str
        :type load_balancers_config:  LoadBalancersConfig
        :type health_check_grace_period: int
        :type health_check_unhealthy_duration_before_replacement: int
        :type ebs_optimized: bool
        :type tenancy: str
        :type iam_role: list[IamRole]
        :type key_pair: str
        :type user_data: str
        :type shutdown_script: str
        :type block_device_mappings: list[BlockDeviceMapping]
        :type network_interfaces: list[NetworkInterface]
        :type tags: list[Tag]
        """
        self.load_balancers_config = load_balancers_config
        self.health_check_type = health_check_type
        self.health_check_grace_period = health_check_grace_period
        self.health_check_unhealthy_duration_before_replacement = health_check_unhealthy_duration_before_replacement
        self.security_group_ids = security_group_ids
        self.monitoring = monitoring
        self.ebs_optimized = ebs_optimized
        self.image_id = image_id
        self.tenancy = tenancy
        self.iam_role = iam_role
        self.key_pair = key_pair
        self.user_data = user_data
        self.shutdown_script = shutdown_script
        self.block_device_mappings = block_device_mappings
        self.network_interfaces = network_interfaces
        self.tags = tags


class LoadBalancersConfig:
    def __init__(self, load_balancers=none):
        """

        :type load_balancers: list[LoadBalancer]
        """
        self.load_balancers = load_balancers


class LoadBalancer:
    def __init__(
            self,
            type=none,
            arn=none,
            name=none,
            target_set_id=none,
            balancer_id=none,
            auto_weight=none,
            az_awareness=none):
        """

        :type type: str
        :type arn: str
        :type name: str
        :type target_set_id: str
        :type balancer_id: str
        :type auto_weight: bool
        :type az_awareness: bool

        """
        self.type = type
        self.arn = arn
        self.name = name
        self.target_set_id = target_set_id
        self.balancer_id = balancer_id
        self.auto_weight = auto_weight
        self.az_awareness = az_awareness


class IamRole:
    def __init__(self, name=none, arn=none):
        """

        :type name: str
        :type arn: str
        """
        self.name = name
        self.arn = arn


class BlockDeviceMapping:
    def __init__(
            self,
            device_name=none,
            ebs=none,
            no_device=none,
            virtual_name=none):
        """

        :type device_name: str
        :type ebs: list[EBS]
        :type no_device: bool
        :type virtual_name: str
        """
        self.device_name = device_name
        self.ebs = ebs
        self.no_device = no_device
        self.virtual_name = virtual_name


class EBS:
    def __init__(
            self,
            delete_on_termination=none,
            encrypted=none,
            iops=none,
            snapshot_id=none,
            volume_size=none,
            volume_type=none,
            kms_key_id=none):
        """

        :type delete_on_termination: bool
        :type encrypted: bool
        :type iops: int
        :type snapshot_id: str
        :type volume_size: int
        :type volume_type: str
        :type kms_key_id: str
        """
        self.delete_on_termination = delete_on_termination
        self.encrypted = encrypted
        self.iops = iops
        self.snapshot_id = snapshot_id
        self.volume_size = volume_size
        self.volume_type = volume_type
        self.kms_key_id = kms_key_id


class Tag:
    def __init__(self, tag_key=none, tag_value=none):
        """

        :type tag_key: str
        :type tag_value: str
        """
        self.tag_key = tag_key
        self.tag_value = tag_value


class NetworkInterface:
    def __init__(
            self,
            delete_on_termination=none,
            device_index=none,
            description=none,
            secondary_private_ip_address_count=none,
            associate_public_ip_address=none,
            groups=none,
            network_interface_id=none,
            private_ip_address=none,
            private_ip_addresses=none,
            subnet_id=none,
            associate_ipv6_address=none):
        """

        :type delete_on_termination: bool
        :type device_index: int
        :type description: str
        :type secondary_private_ip_address_count: int
        :type associate_public_ip_address: bool
        :type groups: list[str]
        :type network_interface_id: str
        :type private_ip_address: str
        :type private_ip_addresses: PrivateIpAddress
        :type subnet_id: str
        :type associate_ipv6_address: str
        """
        self.description = description
        self.device_index = device_index
        self.secondary_private_ip_address_count = secondary_private_ip_address_count
        self.associate_public_ip_address = associate_public_ip_address
        self.delete_on_termination = delete_on_termination
        self.groups = groups
        self.network_interface_id = network_interface_id
        self.private_ip_address = private_ip_address
        self.private_ip_addresses = private_ip_addresses
        self.subnet_id = subnet_id
        self.associate_ipv6_address = associate_ipv6_address


class PrivateIpAddress:
    def __init__(self, private_ip_address=none, primary=none):
        """

        :type private_ip_address: str
        :type primary: bool
        """
        self.private_ip_address = private_ip_address
        self.primary = primary


# endregion

class Roll:
    def __init__(
            self,
            batch_size_percentage=none,
            grace_period=none,
            health_check_type=none,
            strategy=none):
        self.batch_size_percentage = batch_size_percentage
        self.grace_period = grace_period
        self.health_check_type = health_check_type
        self.strategy = strategy


class DetachConfiguration:
    def __init__(
            self,
            instances_to_detach=none,
            should_terminate_instances=none,
            draining_timeout=none,
            should_decrement_target_capacity=none):
        self.instances_to_detach = instances_to_detach
        self.should_terminate_instances = should_terminate_instances
        self.draining_timeout = draining_timeout
        self.should_decrement_target_capacity = should_decrement_target_capacity


class StatefulDeallocation:
    def __init__(
            self,
            should_delete_images=none,
            should_delete_network_interfaces=none,
            should_delete_volumes=none,
            should_delete_snapshots=none):
        self.should_delete_images = should_delete_images
        self.should_delete_network_interfaces = should_delete_network_interfaces
        self.should_delete_volumes = should_delete_volumes
        self.should_delete_snapshots = should_delete_snapshots


class ElastigroupCreationRequest:
    def __init__(self, elastigroup):
        self.group = elastigroup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ElastigroupDeletionRequest:
    def __init__(self, stateful_deallocation):
        self.stateful_deallocation = stateful_deallocation

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ElastigroupUpdateRequest:
    def __init__(self, elastigroup):
        self.group = elastigroup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ElastigroupRollRequest:
    def __init__(self, group_roll):
        self.batch_size_percentage = group_roll.batch_size_percentage
        self.grace_period = group_roll.grace_period
        self.health_check_type = group_roll.health_check_type
        self.strategy = group_roll.strategy

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ElastigroupDetachInstancesRequest:
    def __init__(self, detach_configuration):
        self.should_decrement_target_capacity = detach_configuration.should_decrement_target_capacity
        self.draining_timeout = detach_configuration.draining_timeout
        self.instances_to_detach = detach_configuration.instances_to_detach
        self.should_terminate_instances = detach_configuration.should_terminate_instances

    def toJSON(self):
        return json.dumps(
            self,
            default=lambda o: o.__dict__,
            sort_keys=True,
            indent=4)
